'****************************************************************
'*  Name    : MainTest.BAS                                      *
'*  Author  : John Barrat                                       *
'*  Notice  : Copyright (c) 2004 John Barrat                    *
'*          : All Rights Reserved                               *
'*  Date    : 16/02/2005                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************
#DEFINE TASK_DEBUG
#DEFINE INT_ENABLED
#DEFINE REMRESET_ENABLED
#DEFINE LVD_HANDLER Prog_Shut_Down     
#DEFINE RX_HANDLER ReceiveUSART
#IFDEF RX_HANDLER
   #DEFINE GET_RX_RECORD
#ENDIF
#DEFINE TX_HANDLER TransmitUSART
#DEFINE NO_HSEROUT
#IFDEF INT_ENABLED
   #DEFINE INT0_HANDLER Scheduler_Intr   '
   #DEFINE IOC_HANDLER  KeyScan_Intr     '
   #IFDEF IOC_HANDLER
      #DEFINE TMR1_HANDLER KeyRepTMR	 ' This is the routine to manage Timer1
   	  #DEFINE TMR1_FREQ 50		 	 	 ' Set Timer to 50 Hz (20mS)
   #ENDIF
#ENDIF

MECANIQUE_ICD_REQ = True
BOOTLOADER = On
OPTIMISER_LEVEL = 6

Include "SWCBoardVer3.INC"

Symbol LCD_NoChars = 16
Symbol RB_MinMsgLen	= 5	   	    ' Number of Received Chars before Checking for message	   
Dim MsgArry As String * 32		' General purpose message array for LCD and Serial Messages 

' Set up the maximum tasks for task rate
Symbol ImmdTasks  = 4	   	    ' Must be a power of 2 (i.e. 2, 4, 8 etc)
Symbol Rate1Tasks = 6           ' 2 Hz - No restrictions on the others
Symbol Rate2Tasks = 6           ' 1 Hz
Symbol Rate3tasks = 6           ' 1 Min
Symbol Rate4Tasks = 5           ' 1 Hour
Symbol CoDTasks   = 2           ' Change of Day

' Menu Setup Values
Symbol MnuMTNum = $03           ' The number of Main Menu Items
Symbol MnuMCNum = $04           ' The number of Config Menu Items (1 is for Config Confirmation)

' Variables ------------------------------------------------------------------------------------
Dim DispORptr  As Word	        ' Address of a routine to be run by Execute
Dim TB1 	   As Byte          ' Temporary Byte Variable
Dim TB2		   As Byte
Dim TW1 	   As Word			' Temporary Word Variable
Dim MainMode   As Byte	        ' Bits 4-7 Pump & temp modes, 0 - 2 default disp param (8 Max)
Symbol D_Over_Ride MainMode.3   ' Aliase to Display Override.

Dim OR_Count   As Byte	        ' Holds the number of current procedures forcing Disp Over Ride
Dim RTC_NotSet As Bit	        ' Flag indicating RTC Chip not set up.
Dim Sensor_Acq As Bit	        ' Flag indicating Sensor Acquistion State  True = Acquiring
Dim AdjMode	   As Bit	        ' Flag indicating that displayed value can be altered
Dim PSIMode	   As Bit	        ' PSI = True Bar = False

'Dim Average As Byte

' Prime start up position first time off
SavedMode 	   EData 00         ' Set Normal mode
SvdContrast    EData 00         ' and Contrast to Full

' Set up software stack
REMINDERS OFF
    STACK_SIZE = 20              
REMINDERS On

' Includes -------------------------------------------------------------------------------------
Include "Constants_P+.inc"
Include "Utils_P+.inc"
Include "LCD_P+.inc"             ' 
Include "TaskMan_P+.inc"         ' Task Manager
Include "Int_Ctrl_18F_P+.inc"    ' Interrupt Router
' The following INCs must be added after TASKMAN_INC
Include "RTC_P+.inc"             ' 
Include "I_Schedule_P+.inc"      ' Interrupt Scheduler - must be close to and after RTC_P+.inc
Include "One_Wire_P+.Inc"        ' and after Utils_P+.Inc
Include "Menu_P+.inc"      	     ' and after LCD_INC
Include "RX_Buff_P+.inc"         ' and after Int_Ctrl_18F_P+
Include "Remote_IF_P+.inc"       ' and after RX_Buff_P+
Include "ADC_Max127_P+.inc"      ' 
Include "Pump_Temp_P+.inc"

GoTo Main_Init
'-----------------------------------------------------------------------------------------------
' Updating routines - these routines can be placed on the task list using ADDTask and REMTask
'-----------------------------------------------------------------------------------------------
Disp_Time_L2:
' Displays Time at the end of the bottom line
   TkRow = 2: TkCol = 13
   GoSub DispTime
Return

DispRTC_L2:
' Displays Time and Date on the Bottom Line
   TkRow = 2: GoSub DispTimeDate
Return

'-----------------------------------------------------------------------------------------------
' Menu control subroutines
'-----------------------------------------------------------------------------------------------
' This routine is called on returning from a menu to the default display
RestoreDefaultDisp:
ADDTask [Cycle_Acq, Rate2]		      ' Start Data Acquisition
CLL 			                      ' Clear any previous data
GoSub DefltEnable                     ' Enable the Default Buttons
   ADDTask DispTick, Rate1     	  	  ' Add the routine to toggle the Tick indicator
   ADDTask DispTimeDate, Rate3  	  ' Add the routine to update the display on the minute
   TkRow = 1        		  		  ' set the Row to print Time & Date
   GoSub DispTimeDate	   		      ' refresh display now - we might have to wait for a minute.
UpdateDefault:
Select Case MainMode & $07            ' Choose routine to run from Display Mode bits in MainMode
Case 0
   ADDTask DispTemp, Rate2
   GoSub DispTemp
Case 1
   ADC_Chan_No = 0
   ADDTask Disp_ADC_Values, Rate2
   GoSub Disp_ADC_Values
Case 2
   ADC_Chan_No = 1
   GoSub Disp_ADC_Values
' Add other Cases here
End Select
Return

' This routine is called on leaving the Default display
RemoveDefaultDisp:
   CLL 2                              ' Clear Screen
   REMTask DispTimeDate, Rate3
   REMTask DispTick, Rate1
CancelDefaultDisp:
   CLL 2                              ' Clear line 2
   Select Case MainMode & $07         ' Choose routine to run from Display Mode bits in MainMode
   Case 0
	   REMTask DispTemp, RateAll
   Case 1
	   REMTask Disp_ADC_Values, RateAll	
   Case Else
	' Add other Cases here
End Select
REMTask Cycle_Acq, Rate2		  	  ' Start Data Acquisition
Return

' This routine will invoke the menu control routines according to the Value of ActPtr
ProcMainMenu:
   GoSub CalcPtr                       ' Set pointer value for menu item
   On ActPtr GoToL RestoreDefaultDisp, Menu_Temp, Menu_Pump, Menu_Clock
   GoSub ActionFail [10]
Return
' This routine will invoke the Set control routines according to the Value of ActPtr
ProcMainSet:
   On ActPtr GoToL DefltSet, Set_Temp, Set_Pump, Set_Clock
   GoSub ActionFail [20]
ExitMainSet:
Return
' This routine will invoke the IncDec control routines according to the Value of ActPtr
ProcMainIncDec:
   On ActPtr GoToL DefltIncDec, IncDec_Temp, IncDec_Pump, IncDec_Clock
   GoSub ActionFail [30]
ExitMainIncDec:
Return
   
CancelMainMenu:
   On ActPtr GoToL RemoveDefaultDisp, Can_Menu_Def, Can_Menu_Pump, Can_Menu_Clock
Can_Menu_Def:
Return

' Configuration Menu ---------------------------------------------------------------------------
ProcCfgMenu:
   GoSub CalcPtr                       ' Set pointer value for menu item
'   On ActPtr GoToL Menu_Cfg_Chk, Menu_Cfg_Chk, Menu_Cfg_OW, Menu_Cfg_Exit
   On ActPtr GoToL Menu_Cfg_Chk, Menu_Cfg_Chk, Menu_Cfg_OW, Menu_Cfg_ADC, Menu_Cfg_Exit
   Cls : GoTo ActionFail [11]

ProcCfgSet:   
'   On ActPtr GoToL ExitCfg, Set_Cfg_Chk, Set_Cfg_OW, Set_Cfg_Exit
   On ActPtr GoToL ExitCfg, Set_Cfg_Chk, Set_Cfg_OW, Set_Cfg_ADC, Set_Cfg_Exit
   Cls : GoTo ActionFail [21]

ProcCfgIncDec:   
'   On ActPtr GoToL ExitCfg, IncDec_Cfg_Chk, IncDec_Cfg_OW,  ExitCfg
   On ActPtr GoToL ExitCfg, IncDec_Cfg_Chk, IncDec_Cfg_OW, IncDec_Cfg_ADC, ExitCfg
   Cls : GoTo ActionFail [31]
ExitCfg:
Return

CancelCfgMenu:
'   On ActPtr GoToL Can_Cfg_Def, Can_Cfg_Def, Can_Cfg_Def, Can_Cfg_Def
   On ActPtr GoToL  Can_Cfg_Def, Can_Cfg_Def, Can_Cfg_Def, Can_CfgMnu_ADC, Can_Cfg_Def
Can_Cfg_Def:
Return

' Default button Actions -----------------------------------------------------------------------
' These routines will be invoked when SET, INC or DEC buttons are pressed when the display
' is not in any of the Menu modes and is displaying its default display.
DefltIncDec:
' Add Default Code for Inc/Dec
If DecButton = Pressed Then
   Dec LCD_Contrast
Else
   Inc LCD_Contrast
End If
Set_Contrast:
   LCD_Contrast = LCD_Contrast & $3F ' Keep within visible operating range
GoTo LCD_Adjust_Contrast

DefltSet:
' Add Default Set code here
   GoSub CancelDefaultDisp
   Inc MainMode
   If(MainMode & $07) > 1 Then MainMode = MainMode & $F8
   GoTo UpdateDefault  ' RestoreDefaultDisp
'Return

' This will be called on returning to the default display mode
DefltEnable:
   SetHold = False
   IncDecHold = False
   MnuHold = False
Return

' End of menu control subroutines---------------------------------------------------------------

' Other non-real time routines -----------------------------------------------------------------
Disp_Ver_No:
' Displays the current Software version number
    Cls
    Print mVer, " ", VerNo
Return

' Saves the master control mode in EEPROM
SaveMode:
    EWrite SavedMode, [MainMode]           ' Saves Control mode and default display in EEPROM
Return

SaveContrast:
	EWrite SvdContrast, [LCD_Contrast]     ' Saves current Contrast setting in EEPROM 
Return

' End of Main program Subroutines **************************************************************

Main_Init:

' Board version 3 only
#IFDEF SPCV3 
LCD_Contrast = ERead SvdContrast  ' Retrieve saved contrast setting
GoSub Set_Contrast         		  ' Set up the contrast (Board Version 3 only)
#ENDIF														
DelayMS 200 
GoSub Disp_Ver_No                 ' Show version number

' Get the last updated Pump and Heat Control and Default display settings
MainMode = ERead SavedMode        ' Read the Control Byte and save it in Mainmode
GoSub TaskMan_Init                ' Clear all tasks on startup
GoSub Menu_Init                   ' Reset Menu
DelayMS 2000				 	  ' Leave Version details up for 2 seconds

Cls                               ' Clear screen - Remove Ver number
GoSub CheckRTC                    ' See if clock has previously been set
If RTC_Cont = False Then          ' Clock not set so force user to 
    MenuAct = $10                 ' the Date & Time Set up menu and
    MnuHold = True                ' disable Menu Button until clock setup is complete
	RTC_NotSet = True	   		  ' Set Not Set Flag - used by Menu_Clock
    GoSub Menu_Clock          	  ' Process as a normal menu item - no menu timeout
Else
    GoSub RTC_Init                ' Initialise RTC
    RTC_NotSet = False            ' Clear RTC_NotSet Flag 
    If RTC_Cont = False Then      ' Initialisation failed - Something wrong with 
        GoSub ActionFail [1]      ' the clock chip so set error code and report it
    EndIf
	GoSub ProcMainMenu			  ' Set up the default display
EndIf

GoSub Interrupt_Init              ' Set up Interrupt vectors
GoSub RemoteInit				  ' Start monitoring Serial IP
GoSub USARTBuffInit               ' Set up Serial Port
GoSub OW_Init                     ' Initiate One-Wire drivers
GoSub DefltEnable                 ' enable the default buttons
GoSub PT_Ctrl_Init                ' initiate pump control
GoSub RestoreDefaultDisp		  ' Start the Display
GoTo InitScheduler                ' Start up Scheduler

'Program Shutdown ******************************************************************************

Prog_Shut_Down:
GoSub SaveMode                    ' Save Mode and Display defaults
#IFDEF SPCV3
GoSub SaveContrast                ' Save Contrast
#ENDIF
End

Cycle_Acq:
' When placed on the Task list, routine will cycliclly initiate each sensor reading in 
' turn based on the seconds reading of the RTC - The skips are used to space the readings apart.
' Seconds 0-4:		  		  0			   1  	        2   	    3           4
    On RTCSec & $0F GoToL 	  SkipS,	   SkipS,	    GetTemp,    SkipS,   	SkipS,_
	   		  			  	  Get_ADC_Val, SkipS,		SkipS, 		SkipS, 		SkipS
' Seconds 5-9:     	  	      5			   6		    7		  	8		   	9		   

SkipS:
Return

GetADCVal:
  GoSub Get_ADC_Val [ADC_Chan_No]
Return

GetTemp:
    OW_Dev_Num = 0
	ADDTask OW_Req_Single_Temp, Rate2   ' Set up Acquisition request at next second count
Return

DispTemp:
 	MsgArry = mTemp + " "
    Cursor 2,1: GoSub OW_Display_Temp
Return

' Config Check Routines ************************************************************************
' These routines ask the user to confirm they want to continue with config or return to normal 
Menu_Cfg_Chk:                    
	Print mConfig                       ' Print "Config" on top line
	Print At 2,1, mYes              	' on bottom line print "Yes" above the SET
	Print At 2,5, mNo			    	' "No" over DEC button
    SetHold = False: IncDecHold = False ' Allow Set and IncDec buttons
    MnuHold = True                  	' Disable use of Menu button
Return

IncDec_Cfg_Chk:
    If DecButton = Pressed Then Set_Cfg_Exit ' User wants to leave Config
Return

Set_Cfg_Chk:
    MnuHold = False						' Allow use of Menu button
    REMTask DecMenuTimer, Rate1         ' Remove Display timeout and  
	REMTask Cycle_Acq, Rate2            ' cycle Acquisition tasks from task list
	Cls	                                ' Clear Screen
 	MenuAct = $20                     	' Skip the first Menu Item (Check Config)
  	GoTo ProcCfgMenu                  	' and process the first config item

Show_Cfg_OK:
	Print At 2,1,  mOK 	   		    	' Move to bottom line Print "OK" adjacent to SET button
Show_Cfg_Arrow:
    Print At 2,16, mChevRh			    ' Set cursor over the "Menu" Button and print ">"
Return

' Config Exit Routines *************************************************************************
' Offers the option to exit - Alternatively Menu button takes user back to first Config Menu 
Menu_Cfg_Exit:
	Print mExit, " ", mConfig           ' Print "Exit Config" above 
	SetHold = False:  MnuHold = False	' Allow Set button and Menu Button  
	IncDecHold = True					' disable Inc/Dec buttons
	GoTo Show_Cfg_OK                    ' Print "OK" above Set Button and Arrow above Menu
	                                    ' and return through Show_Cfg_OK

Set_Cfg_Exit:                           ' User wants to leave Config mode
    MenuAct = $00: ConfigMode = False   ' Reset MenuAct to normal, clear Config mode Flag 
	ADDTask Cycle_Acq, Rate2            ' cycle Acquisition tasks from task list
	GoTo ProcMainMenu	  				' and Return to the default display

' MAIN MENU ROUTINES ***************************************************************************

' Temperature control Menu *********************************************************************
Menu_Temp:
    Print mHeating                       ' Print "Heating "
	TB1 = 0: AdjMode = False		     ' Point TB1 to first display param and AdjMode to False
    Cursor 2,1: GoSub Disp_Htg_Param   	 ' Display Current Heating Mode
    SetHold = False:IncDecHold = False   ' Always allow Set and IncDec buttons     
	If HeatOnOff = SwOff Then            ' Set up the submenu count for current heating mode
       SubMenuCnt = 1                    ' If off the only options are to change mode
	Else
       If HeatCtrlMode = Manual Then SubMenuCnt = 4 ' Mode, Target Temp, Rate
       If HeatCtrlMode = Timed  Then SubMenuCnt = 6 ' Mode, Tgt Tmp, Rate, Day On, Day Off
	EndIf
Return

Set_Temp:
	If TB1 = 1 Then Return			     ' Set Doesn't apply to Pool Temperature
	If TB1 = 3 Then Return               ' or Heating Rate display
    CLL 2	   	 			  	 		 ' Clear Bottom line
    If AdjMode = False Then           	 ' If we are moving into Adjust mode  
       AdjMode = True	 			  	 ' Allow parameter to be adjusted.
       Print mSet, " "  	 			 ' Prefix line with "Set "
       If TB1 = 0 Then GoSub Str_Heat_Mode' set up character string to display heat mode
	Else
	   If TB1 = 0 Then                   ' If we are doing a mode change
	   	  Cls                   	 	 ' Clear screen and 
	   	  GoSub SetNewHeatMode		  	 ' Set new Heating mode
	   	  GoSub SaveMode			  	 ' Save new mode
          GoTo Menu_Temp              	 ' start over
	   EndIf
	   AdjMode = False                	 ' Come out of Adjust Mode, 
	   GoSub Save_Temp_Ctrl			  	 ' and Save current parameters
	EndIf
	GoTo Disp_Htg_Param		  	  	  	 ' Displaying Heating params

IncDec_Temp:
    CLL 2		  			     	 	 ' Clear Bottom Line
    If AdjMode = True Then            	 ' In adjust mode - IncDec will adjust parameter values
       GoSub SetIncDecDir	          	 ' Set up direction to adjust param
       Print mSet  	 					 ' Prefix line with "Set "
	   Select Case TB1
 	   Case 0: GoSub Step_Heat_Mode   	 ' Go to next heating mode
       Case 2: GoSub Step_Tgt_Temp	 	 ' Adjust target temperature
	   Case 4: GoSub StepTgtDay [TgtDayOn], TgtDayOn       ' Adjust DayOn Day
       Case 5: GoSub StepTgtDayCont [TgtDayOff], TgtDayOff ' Adjust DayOff Day - 8 is Continuous
       End Select
	Else                                 ' Not in adjust mode - INCDEC will move between params
       GoSub IncDecSubMenu               ' Move to next or previous paramater
       TB1 = MenuAct & $0F           	 ' Set current Parameter index in TB1
    EndIf
Disp_Htg_Param:
    On TB1 GoSub Str_Heat_Mode, Str_Pool_Temp, Str_Tgt_Temp,_
       	   		 Str_Pool_Rate, Str_Day_On,    Str_Day_Off
       	   		 
    Print MsgArry             		     ' Print the information 
Return

SetIncDecDir:
	If IncButton = Pressed Then   	  	 ' Set up the direction to make the adjustments
	   SetIncr = Incr			  	  	 ' Increment parameter
	Else
	   SetIncr = ~ Incr		  	  	 	 ' Decrement parameter
	EndIf 	  
Return

' Pump control Menu ****************************************************************************
Menu_Pump:
    Print mPump                          ' Print "Pump "
	TB1 = 0: AdjMode = False		     ' Point TB1 to first display param and AdjMode to False
    GoSub Str_Pump_Mode   	 			 ' Get current pump "Mode" as string and
	Print At 2,1, MsgArry	  			 '  on bottom line
    SetHold = False: IncDecHold = False  ' Always allow Set and IncDec buttons  
    'Set up the submenu count according to the Pump mode we are in
	SubMenuCnt = 1                       ' Default count is 1 - to change mode
    If PumpOnOff = SwOn Then          	 ' but if pump is on and in timed mode 
       If PumpCtrlMode = Timed Then   	 ' add in the Program display and edit submenus
	   	  SubMenuCnt = 8              	 ' Set Mode & Timed Setup per day
   	   	  TB2 = RTCWDay - 1		      	 ' TB2 holds Offset day no of program day starting at today
       	  PT_TSlot = 0	   			  	 ' TC_TSlot holds current timeslot (for editing)
	   EndIf	  
	EndIf    
Return

Set_Pump:
    If AdjMode = False Then           	 ' If we are moving into Adjust mode prefix 
       CLL 2	   	 			  	 	 ' Clear Bottom line and
       AdjMode = True	 			  	 ' Allow parameter to be adjusted.
       If TB1 = 0 Then                	 ' We are in the Mode sub menu so
       	  Print mSet  	 				 ' Prefix line with "Set "
	   	  GoSub Str_Pump_Mode 		  	 ' Display current mode of Pump
 	   Else                           	 ' Set up to edit program for current day
       	  PT_TSlot = 0				  	 ' Set up the pointer to the first 2Hr slot
       	  GoSub GetNextProg
      	  GoSub Disp_Timed_Prog_Ed 	 	 ' and get the program for editing
       	  GoSub PT_Highlight           	 ' Highlight the current 2Hr slot setting
       EndIf
	Else
	   If TB1 = 0 Then                	 ' If we are doing a mode change
	      GoSub SaveMode			  	 ' Save new mode in EEProm
	      GoSub SetNewMode		  	 	 ' Activate mode change
	   	  Cls                   	 	 ' Clear screen and 
          GoTo Menu_Pump              	 ' start over
	   Else
		  Cursor 2,1 	   	 	  	 	 ' Csr to start of line 2
	      If PT_TSlot = 12 Then       	 ' User pressed save so 
	         AdjMode = False          	 ' come out of Adjust Mode, 
	         GoSub PT_Update_Prog 		 ' save new value in Program array
		     GoSub Disp_Timed_Prog    	 ' and redisplay current days program
		     GoSub SavePumpProg	[PT_Day] ' and Save current parameters in EEProm
		     GoSub Timed_Pump_Ctrl   	 ' Force timing control to update immediately
	      Else
	         GoSub ToggleTSlot           ' Change the state of the TSlot
    	  	 GoSub Disp_Timed_Prog_Ed    ' and redisplay current days program
    	  	 GoSub PT_Highlight          ' Highlight current slot
		  EndIf
	   EndIf
	EndIf
	Print MsgArry
Return

IncDec_Pump:
    CLL 2		  			  	 		 ' Clear Bottom Line
    GoSub SetIncDecDir             	  	 ' Set up direction to adjust slot or day
    If AdjMode = True Then            	 ' In adjust mode - IncDec will adjust parameter values
       If TB1 = 0 Then
       	  Print mSet  	 				 ' Prefix line with "Set "
          GoSub Step_Pump_Mode  	  	 ' Go to next Pump mode
   	      GoSub Str_Pump_Mode  	  	 	 ' and display it
       Else                           	 ' Move through individual 2 Hr slots
		  GoSub IncDecTSlot			  	 ' Set up the pointer to the next 2Hr slot
	      If PT_TSlot = 12 Then       	 ' Been through all slots so offer option to save
		     MsgArry = mOK 				 ' Print OK over SET button
		  Else
	         GoSub Disp_Timed_Prog_Ed  	 ' display pump program for editing
		  	 GoSub PT_Highlight			 ' Set State and Highlight the current slot position
          EndIf
	   EndIf       
    Else                              	 ' Not in adjust mode - INCDEC moves between prog days
       GoSub IncDecSubMenu            	 ' Move to next or previous paramater
       TB1 = MenuAct & $0F            	 ' Set current Parameter index in TB1
       If TB1 > 0 Then                	 ' If not in Mode menu
          PT_Day = TB1 + TB2      	  	 ' Add in offset for today and
          If PT_Day > 7 Then PT_Day = PT_Day - 7' 
          GoSub GetNextProg
		  GoSub Disp_Timed_Prog		  	 ' and display the program		
       Else
   	      GoSub Str_Pump_Mode		  	 ' display current pump mode
	   EndIf
	EndIf
	Print MsgArry
Return

Can_Menu_Pump:                           ' menu being cancelled so
'    REMTask PT_Flash, Rate1  	 	 	 ' Remove the Flash task from the task list						  
Return

Disp_Timed_Prog_Ed:
   GoSub PT_Set_EdChar
   GoTo Disp_Prog
' Displays the program for day number PT_Day 
Disp_Timed_Prog:                         '
   GoSub PT_Set_VwChar
Disp_Prog:
   Clear MsgArry
   GoSub StrDayName [PT_Day]            ' Get the current day of week and as "DDD"
   MsgArry = MsgArry + " "              ' Add in a space
   GoSub Str_Pump_Prog                  ' Show pump progam as series of 2 Hour steps
Return

' Clock Setup Routines *************************************************************************
Menu_Clock:
' Title top line and display Daylight saving option and show the current Time in the RTC Array
    Print mSet, " ", mClock             ' Title "Set Clock"
    SetHold = False: TB1 = 0	 		' Allow SET button, Reset TB1
    IncDecHold = False	   				' Allow INC/DEC buttons
    If RTC_NotSet = False Then          ' Test to see if RTC Set up
       TkRow = 2: TkCol = 13            ' place Time at end of bottom row
       ADDTask DispTick, Rate1          ' Add updating task and
       ADDTask Disp_Time_L2, Rate3      ' updating Time on Bottom line
	EndIf
    GoSub TSetup: GoSub Disp_DS_Mode    ' Get Time/Date ready for editing & Display DS State       	 	   					' 
	SubMenuCnt = 7						' Set up the Submenu Count 
Return

Disp_DS_Mode:
    CLL 2                               ' Clear bottom line
    Print mDS, " "                 		' and print "D-S "
	If DSTime = True Then               ' determine current D-S state 
        Print mOn
    Else 
        Print mOff                 	    
	EndIf
    GoSub Disp_Time_L2                  ' Display updated time imediately
Return

Set_Clock:
    GoSub IncSubMenu					' Move pointer to the next sub menu
    ' First depression of SET button starts the Clock Update cycle
    TB1 = MenuAct & $0F                 ' Put Submenu number in Temp Var TB1
    If TB1 = $1 Then
        GoSub Can_Menu_Clock            ' remove updating display from task list
        Cls                             ' Clear LCD
        Print mSet                      ' Display "Set " 
        GoSub TSetup                    ' Get TimeDate ready for editing
        IncDecHold = False              ' Allow Use of IncDec buttons
    EndIf    
    If TB1 = $6 Then 
        GoSub RTCUpDate                 ' Reached the end of Set Clock cycle so update RTC       
        Cursor 1,1					    ' Cursor to top line to report
		If RTC_Cont = False Then        ' Test to see if update successful
            GoSub ActionFail [1]        ' No Set Error code to RTC update problem and Report
            MnuHold = True              ' Try again
        Else
            CLL 1: GoSub ActionOK       ' Clear top line OK to Report Update Completed
            If RTC_NotSet = True Then Main_Init ' If 1st time clock set restart as RTC now set
            MnuHold = False             ' Allow use of menu button
        EndIf
        MenuAct = MenuAct & $F0         ' Reset MenuAct
        Cls
        GoTo Menu_Clock                 ' Back to top menu
    Else
        MsgArry = " "
        GoSub StrRTCUnit [TB1]          ' Use TB1 pointer to index the parameter and display
        Print At 1,4, MsgArry           ' "Hours", "Mins ", "Day  ", "Month", "Year "
        Cursor 2,1: GoSub DispTimeDate2 ' Show edited time/date on line 2
    EndIf
Return

IncDec_Clock:
    If TB1 = 0 Then	  					' Still in DS mode so
       GoSub ToggleDS    				' Change Daylight saving
       GoSub RTCUpDate                  ' Save new setting in RTC
       GoSub Disp_DS_Mode               ' and redisplay setting
	Else
   	   GoSub ClkAdjust                  ' Push return address and jump to routine
       CLL 2: GoSub DispTimeDate2       ' Display updated time & date
	EndIf	
Return
ClkAdjust:      
    RTC_Cont = DecButton                ' Set the Inc/Dec Direction 
    ' branch to clock adjust routines determined by MenuAct returning through selected routine  
    On TB1 - 1 GoToL StepHour, StepMin, StepDay, StepMonth, StepYear
    GoTo BeepErr ' If we have reached here there is a error Return through the BeepErr

Can_Menu_Clock:
    REMTask Disp_Time_L2, Rate3         ' Remove Time Display task used by Clock menu
    REMTask DispTick, Rate1             ' and the updating tick indicator
Return

' Set Up and Configuration Menus ***************************************************************

' Config 1-Wire Set Up Routines ****************************************************************
Menu_Cfg_OW:
	Print mOWDev, mSetUp                ' Display "OWDev Set-up" on top line
	SetHold = False: IncDecHold = True	' Allow SET button, Disable IncDec
	SubMenuCnt = 3                      ' Set up the number of OW Sub Menus
	GoTo Show_Cfg_OK 					' Print "OK" above Set Button and Arrow above Menu
	                                    ' Return through the Show_Cfg_OK Sub
Set_Cfg_OW:
    GoSub IncSubMenu		  			' Advance the Submenu value
    TB1 = MenuAct & $0F					' Set up pointer to next activity
	If TB1 = 1 Then	                    ' User has inserted a device to interrogate
	    GoSub OW_Find_Dev_ID 			' Try and find it - Returns ID in OW_ID, MsgIdx 
	    If OW_Dev_Num & $F7 = $F7 Then  ' and OW_Dev_Num with status - Test for Errors
	       If OW_Dev_Num = $FE Then     ' Unexpected device detected so
		   	  Cls
 			  GoSub OW_Disp_ID   		' show device ID
 			  Print MsgArry
		   EndIf   
		   If OW_Dev_Num = $FF Then     ' No device detected so
 			  Print At 1, 7, MsgArry 	' Print "Not Found" on Top line
 	  	   EndIf	  
		   CLL 2      	   	 			' Clear bottom line and
		   GoSub Show_Cfg_OK 			' Print "OK" above Set Button
        Else                            ' No error so let user assign the function device
		   CLL 2: Print MsgArry  		' Print Device Family Name returned and 
           GoSub OW_Disp_ID: Print At 1,1, MsgArry ' Display Device ID on top line
		   GoSub NextDev  				' first Device function option on bottom line
		   IncDecHold = False 			' Allow INC/DEC buttons so user can choose function
		EndIf
	EndIf  
	If TB1 = 2 Then                     ' When SET is pressed again
 	   If (OW_Dev_Num & $F7) < $F7 Then ' Check if there has been any errors
 	      GoSub OW_Save_ID				' No, so save ID into EEPROM
 	      CLL 1
	      GoSub ActionOK      			' and report Updated
       Else
   	      GoSub BeepErr                 ' Indicate error
	   EndIf
 	   MenuAct = MenuAct & $F0			' Reset Sub Menu and invoke OW Main Menu
 	   Cls
 	   GoTo Menu_Cfg_OW	    			' go back to the O-W Set up menu
    EndIf
    GoTo Show_Cfg_Arrow             	' Place arrow over the Menu key, return through Sub

IncDec_Cfg_OW:                              
    GoSub OW_Next_Funct                 ' Cycles through possible function 
NextDev:
	GoSub OW_Dev_Name [OW_Dev_Num]      ' options for current device, 
	Print At 2,6, MsgArry               ' displays them on the bottom line 
Return                                  ' and returns.

' Config ADC Offsets ***************************************************************************
Menu_Cfg_ADC:
	Print At 1,1, mSet, " ", mADC          	' Display "Set ADC" on top Line
	ADC_Chan_No = 7
    IncDecHold = False     					' Enable SET & IncDec buttons
    SubMenuCnt = 4                          ' Set up the ADC submenu Count
	GoSub IncDec_Cfg_ADC                    ' Set up for first channel
	ADDTask Disp_Raw_ADC, Rate2           	' Add ADC display routine to Rate 1 task list
Return

Set_Cfg_ADC:
    GoSub IncSubMenu		  				' Advance the Submenu value
	Select Case MenuAct & $0F               ' Set up pointer to next activity
	
	Case 1 
       GoSub ADC_Get_Min			  		' Gets the Min value
	   If ADC_OK = False Then Acqu_Failed	' Check Acquisition OK
  	   GoTo ADC_Cfg_Prompt         			' update prompt
	Case 2                                  ' Get the Max Value
	   GoSub ADC_Get_Max                    ' Already have min value so get Max value and
	   If ADC_OK = False Then Acqu_Failed   ' Indicate and return	   
	   REMTask Disp_Raw_ADC, Rate2        	' Remove ADC display routine from Rate 1 task list
	   Print At 1,9, mCh, " ", DEC1 ADC_Chan_No, " " , mOK, "?"
	   Print At 2,1, "k=", HEX4 ADCk," m=", HEX4 ADCm ' and show correction values
	Case 3                                  ' Save new values
	   GoSub ADC_Save_Scaling		 	    ' Save new values in EEPROM
       CLL 2: GoSub ActionOK           		' Report Updated on bottom line
	End Select
	GoTo Show_Cfg_Arrow                     ' Place arrow over the Menu key, return through Sub
Acqu_Failed:
    CLL 2: GoTo BeepErr

IncDec_Cfg_ADC:
    MenuAct = MenuAct & $F0					' Reset Sub Menu
    GoSub ADC_Next_Chan                     ' Increment ADC_Chan_No
	GoSub ADC_Get_Real_Values             	' and Real Maximum Value for next Channel
	GoSub ADC_Cfg_Prompt                    ' Display Prompt
	GoSub Disp_Raw_ADC		  				' Display Raw ADC Value now.                  
Return

Can_CfgMnu_ADC:
    REMTask Disp_Raw_ADC, Rate2
Return

Disp_Raw_ADC:
	Print At 2,1, mCh, " ", DEC1 ADC_Chan_No' Print "Ch n" - where n is current channel number
 	GoSub Get_Raw_ADC_Val	  				' Get current uncorrected reading
	If ADC_OK = True Then                   ' If ADC Acquisition OK
	   SetHold = False						' Enable Set
	   Print At 2,6, HEX4 ADC_Val
	Else                                    ' if not OK
	   Print At 2,6, mNA                    ' Print "N/A"
	   SetHold = True                       ' Disable SET Button
	EndIf 
	GoTo Show_Cfg_Arrow                     ' Place arrow over the Menu key, Return through Sub

ADC_Cfg_Prompt:
	If (MenuAct & $0F) = 0 Then 
		MsgArry = mMin
    	TW1 = ADCMiR  'ADC_Values[MaxVal] - ADC_Values[Range]' Calc minimum real Value
	Else
		MsgArry = mMax
		TW1 = ADCMxR 'ADC_Values[MaxVal]            ' Insert Max real Value            
	EndIf 
	Print At 1,9, MsgArry, " ", DEC4 TW1    ' Print on top Line, Print Real value,
Return
Stop

PName:  CData "MainTest ",0
VerNo:  CData "1.0.1.3",0
Include "SPC_Messages_P+.bas"

